const path = require('path')
const packageName = require('./package.json').name;
//路径转换方法
function resolve(dir) {
	return path.join(__dirname, dir)
}
const CompressionPlugin = require('compression-webpack-plugin') // compression-webpack-plugin插件需要npm安装
const UglifyJsPlugin = require('uglifyjs-webpack-plugin');
const IS_PROD = process.env.NODE_ENV === 'production'; // 是否为生产环境(打包/build后)

const api= 'www.saas123.com';
const SJCDN = {
	css: [
		'http://'+api+'/cdnjs/antd.min.css',
		'http://saas123.com/cdnjs/github-markdown.css',
	],
	js: [
		'http://'+api+'/cdnjs/vue.min.js',
		'http://'+api+'/cdnjs/axios.min.js',
		'http://'+api+'/cdnjs/vuex.min.js',
		'http://'+api+'/cdnjs/vue-router.min.js',
		'http://'+api+'/cdnjs/vue-i18n.min.js',
		'http://'+api+'/cdnjs/moment-with-locales.min.js',
		'http://'+api+'/cdnjs/antd-with-locales.min.js',
		'http://'+api+'/cdnjs/marked.min.js',
		//'http://'+api+'/cdnjs/highlight.min.js',
	],
}
module.exports = {
	publicPath: IS_PROD ? '/' : '/', // 公共路径
	outputDir: 'dist', // 'dist', 生产环境构建文件的目录
	assetsDir: 'static', // 相对于outputDir的静态资源(js、css、img、fonts)目录
	indexPath: 'index.html', // 相对于打包路径index.html的路径
	filenameHashing: true, //文件名中包含了 hash 以便更好的控制缓
	lintOnSave: !IS_PROD, // 设置eslint报错时停止代码编译
	runtimeCompiler: true, // 是否使用包含运行时编译器的 Vue 构建版本
	productionSourceMap: !IS_PROD, // 不需要生产环境的 source map（减小dist文件大小，加速构建）
	integrity: true, //如果你构建后的文件是部署在 CDN 上的，启用该选项可以提供额外的安全性
	pwa: {}, // 向 PWA 插件传递选项。
	parallel: require("os").cpus().length > 1, // 是否为 Babel 或 TypeScript 使用 thread-loader。该选项在系统的 CPU 有多于一个内核时自动启用，仅作用于生产构建。
	devServer: { //开发服务
		host: '0.0.0.0', // 匹配本机IP地址(默认是0.0.0.0)
		port: 8090, // 开发运行端口号
		open: true, // npm run serve后自动打开页面
		hotOnly: true, // 热更新
		disableHostCheck: true, //取消头部域名的检测
		overlay: { // 让浏览器 overlay 同时显示警告和错误
			warnings: false,
			errors: false
		},
		headers: {
			'Access-Control-Allow-Origin': '*',
		},
	},
	chainWebpack: config => {
		// 第1个参数：别名，第2个参数：路径（设置路径别名）
		config.resolve.alias
			.set('@', resolve('src'))
		// 生产环境注入cdn
		config.plugin('html')
			.tap(args => {
				args[0].cdn = SJCDN;
				return args;
			});
		config.module
			.rule('md')
			.test(/\.md$/)
			.use('text-loader')
			.loader('text-loader')
			.end();
		if (IS_PROD) {
			// 移除 prefetch 插件(针对生产环境首屏请求数进行优化)
			config.plugins.delete('prefetch')
			// 移除 preload 插件(针对生产环境首屏请求数进行优化)
			config.plugins.delete('preload')
			// 压缩代码
			config.optimization.minimize(true);
			// 分割代码
			config.optimization.splitChunks({
				chunks: 'all'
			})
			//打包分析, 打包之后自动生成一个名叫report.html文件(可忽视)
//			config.plugin("webpack-report").use(BundleAnalyzerPlugin, [{
//				analyzerMode: "static"
//			}]);
		}

	},
	// 配置打包 js、css文件为.gz格式，优化加载速度
	configureWebpack: config => {
		// 把子应用打包成 umd 库格式  packageName改成跟主应用中name注册的一致
		config.output.library = `${packageName}-[name]`
		config.output.libraryTarget = 'umd'
		config.output.jsonpFunction = `webpackJsonp_${packageName}`
		//用cdn方式引入,前面是项目内引入的，后面是挂载到window对象下的
		config.externals = {
			'vue': 'Vue',
			'vuex': 'Vuex',
			'vue-router': 'VueRouter',
			'axios': 'axios'
		}
		if (IS_PROD) {
			// 为生产环境修改配置...
			config.plugins.push(
				//生产环境自动删除console
				new UglifyJsPlugin({
					uglifyOptions: {
						compress: {
							drop_console: true,
							drop_debugger: false,
							pure_funcs: ["console.log"] //移除console
						},
					},
					sourceMap: false,
					parallel: true,
				})
			);
			config.plugins.push(
				new CompressionPlugin({
					test: /\.js$|\.css/, // 匹配文件
					threshold: 10240, // 超过10kB的数据进行压缩
					deleteOriginalAssets: false // 是否删除原文件 （原文件也建议发布到服务器以支持不兼容gzip的浏览器）
				})
			)
		}
	},
	css: {
		// 是否使用css分离插件 ExtractTextPlugin
		// extract: false,
		// 开启 CSS source maps?
		sourceMap: false,
		// css预设器配置项
		loaderOptions: {},
		// 启用 CSS modules for all css / pre-processor files.
		requireModuleExtension: false,
	},
}
